@@ -17,13 +17,13 @@
 #define MAX2(x, y) ((x) > (y) ? (x) : (y))
 #define MAX3(x, y, z) ((x) > (y) ? ((x) > (z) ? (x) : (z)) : ((y) > (z) ? (y) : (z)))
 
-/** 
- * alg_locate_center_size 
- *      Locates the center and size of the movement. 
+/**
+ * alg_locate_center_size
+ *      Locates the center and size of the movement.
  */
 void alg_locate_center_size(struct images *imgs, int width, int height, struct coord *cent)
 {
-    unsigned char *out = imgs->out;
+    unsigned char *out = imgs->img_motion.image_norm;
     int *labels = imgs->labels;
     int x, y, centc = 0, xdist = 0, ydist = 0;
 
@@ -65,13 +65,13 @@ void alg_locate_center_size(struct images *imgs, int width, int height, struct c
         cent->x = cent->x / centc;
         cent->y = cent->y / centc;
     }
-    
+
     /* Now we find the size of the Motion. */
 
     /* First reset pointers back to initial value. */
     centc = 0;
     labels = imgs->labels;
-    out = imgs->out;
+    out = imgs->img_motion.image_norm;
 
     /* If Labeling then we find the area around largest labelgroup instead. */
     if (imgs->labelsize_max) {
@@ -90,7 +90,7 @@ void alg_locate_center_size(struct images *imgs, int width, int height, struct c
 
                     centc++;
                 }
-            }    
+            }
         }
 
     } else {
@@ -109,18 +109,18 @@ void alg_locate_center_size(struct images *imgs, int width, int height, struct c
 
                     centc++;
                 }
-            }    
+            }
         }
 
     }
-    
+
     if (centc) {
         cent->minx = cent->x - xdist / centc * 2;
         cent->maxx = cent->x + xdist / centc * 2;
-        /* 
+        /*
          * Make the box a little bigger in y direction to make sure the
          * heads fit in so we multiply by 3 instead of 2 which seems to
-         * to work well in practical. 
+         * to work well in practical.
          */
         cent->miny = cent->y - ydist / centc * 3;
         cent->maxy = cent->y + ydist / centc * 2;
@@ -151,31 +151,31 @@ void alg_locate_center_size(struct images *imgs, int width, int height, struct c
     cent->miny += cent->miny % 2;
     cent->maxx -= cent->maxx % 2;
     cent->maxy -= cent->maxy % 2;
-    
+
     cent->width = cent->maxx - cent->minx;
     cent->height = cent->maxy - cent->miny;
-    
+
     /*
      * We want to center Y coordinate to be the center of the action.
      * The head of a person is important so we correct the cent.y coordinate
-     * to match the correction to include a persons head that we just did above. 
+     * to match the correction to include a persons head that we just did above.
      */
     cent->y = (cent->miny + cent->maxy) / 2;
-    
+
 }
 
 
-/** 
- * alg_draw_location 
- *      Draws a box around the movement. 
+/**
+ * alg_draw_location
+ *      Draws a box around the movement.
  */
 void alg_draw_location(struct coord *cent, struct images *imgs, int width, unsigned char *new,
                        int style, int mode, int process_thisframe)
 {
-    unsigned char *out = imgs->out;
+    unsigned char *out = imgs->img_motion.image_norm;
     int x, y;
 
-    out = imgs->out;
+    out = imgs->img_motion.image_norm;
 
     /* Debug image always gets a 'normal' box. */
     if ((mode == LOCATE_BOTH) && process_thisframe) {
@@ -191,7 +191,7 @@ void alg_draw_location(struct coord *cent, struct images *imgs, int width, unsig
         }
 
         for (y = cent->miny; y <= cent->maxy; y++) {
-            int width_minx_y = cent->minx + y * width; 
+            int width_minx_y = cent->minx + y * width;
             int width_maxx_y = cent->maxx + y * width;
 
             out[width_minx_y] =~out[width_minx_y];
@@ -211,7 +211,7 @@ void alg_draw_location(struct coord *cent, struct images *imgs, int width, unsig
         }
 
         for (y = cent->miny; y <= cent->maxy; y++) {
-            int width_minx_y = cent->minx + y * width; 
+            int width_minx_y = cent->minx + y * width;
             int width_maxx_y = cent->maxx + y * width;
 
             new[width_minx_y] =~new[width_minx_y];
@@ -228,19 +228,19 @@ void alg_draw_location(struct coord *cent, struct images *imgs, int width, unsig
         for (y = cent->y - 10; y <= cent->y + 10; y++) {
             new[cent->x + y * width] =~new[cent->x + y * width];
             out[cent->x + y * width] =~out[cent->x + y * width];
-        }       
+        }
     }
 }
 
 
-/** 
- * alg_draw_red_location 
+/**
+ * alg_draw_red_location
  *          Draws a RED box around the movement.
  */
 void alg_draw_red_location(struct coord *cent, struct images *imgs, int width, unsigned char *new,
                            int style, int mode, int process_thisframe)
 {
-    unsigned char *out = imgs->out;
+    unsigned char *out = imgs->img_motion.image_norm;
     unsigned char *new_u, *new_v;
     int x, y, v, cwidth, cblock;
 
@@ -248,7 +248,7 @@ void alg_draw_red_location(struct coord *cent, struct images *imgs, int width, u
     cblock = imgs->motionsize / 4;
     x = imgs->motionsize;
     v = x + cblock;
-    out = imgs->out;
+    out = imgs->img_motion.image_norm;
     new_u = new + x;
     new_v = new + v;
 
@@ -266,7 +266,7 @@ void alg_draw_red_location(struct coord *cent, struct images *imgs, int width, u
         }
 
         for (y = cent->miny; y <= cent->maxy; y++) {
-            int width_minx_y = cent->minx + y * width; 
+            int width_minx_y = cent->minx + y * width;
             int width_maxx_y = cent->maxx + y * width;
 
             out[width_minx_y] =~out[width_minx_y];
@@ -279,7 +279,7 @@ void alg_draw_red_location(struct coord *cent, struct images *imgs, int width, u
         int width_maxy = width * cent->maxy;
         int cwidth_miny = cwidth * (cent->miny / 2);
         int cwidth_maxy = cwidth * (cent->maxy / 2);
-        
+
         for (x = cent->minx + 2; x <= cent->maxx - 2; x += 2) {
             int width_miny_x = x + width_miny;
             int width_maxy_x = x + width_maxy;
@@ -305,9 +305,9 @@ void alg_draw_red_location(struct coord *cent, struct images *imgs, int width, u
         }
 
         for (y = cent->miny; y <= cent->maxy; y += 2) {
-            int width_minx_y = cent->minx + y * width; 
+            int width_minx_y = cent->minx + y * width;
             int width_maxx_y = cent->maxx + y * width;
-            int cwidth_minx_y = (cent->minx / 2) + (y / 2) * cwidth; 
+            int cwidth_minx_y = (cent->minx / 2) + (y / 2) * cwidth;
             int cwidth_maxx_y = (cent->maxx / 2) + (y / 2) * cwidth;
 
             new_u[cwidth_minx_y] = 128;
@@ -329,7 +329,7 @@ void alg_draw_red_location(struct coord *cent, struct images *imgs, int width, u
         }
     } else if (style == LOCATE_REDCROSS) { /* Draw a red cross on normal images. */
         int cwidth_maxy = cwidth * (cent->y / 2);
-        
+
         for (x = cent->x - 10; x <= cent->x + 10; x += 2) {
             int cwidth_maxy_x = x / 2 + cwidth_maxy;
 
@@ -338,8 +338,8 @@ void alg_draw_red_location(struct coord *cent, struct images *imgs, int width, u
         }
 
         for (y = cent->y - 10; y <= cent->y + 10; y += 2) {
-            int cwidth_minx_y = (cent->x / 2) + (y / 2) * cwidth; 
-            
+            int cwidth_minx_y = (cent->x / 2) + (y / 2) * cwidth;
+
             new_u[cwidth_minx_y] = 128;
             new_v[cwidth_minx_y] = 255;
         }
@@ -366,7 +366,7 @@ void alg_noise_tune(struct context *cnt, unsigned char *new)
     unsigned char *smartmask = imgs->smartmask_final;
 
     i = imgs->motionsize;
-            
+
     for (; i > 0; i--) {
         diff = ABS(*ref - *new);
 
@@ -385,7 +385,7 @@ void alg_noise_tune(struct context *cnt, unsigned char *new)
 
     if (count > 3)  /* Avoid divide by zero. */
         sum /= count / 3;
-    
+
     /* 5: safe, 4: regular, 3: more sensitive */
     cnt->noise = 4 + (cnt->noise + sum) / 2;
 }
@@ -407,7 +407,7 @@ void alg_threshold_tune(struct context *cnt, int diffs, int motion)
 
     for (i = 0; i < THRESHOLD_TUNE_LENGTH - 1; i++) {
         sum += cnt->diffs_last[i];
-        
+
         if (cnt->diffs_last[i + 1] && !motion)
             cnt->diffs_last[i] = cnt->diffs_last[i + 1];
         else
@@ -455,8 +455,8 @@ typedef struct {
 /**
  * iflood
  *
- */ 
-static int iflood(int x, int y, int width, int height, 
+ */
+static int iflood(int x, int y, int width, int height,
                   unsigned char *out, int *labels, int newvalue, int oldvalue)
 {
     int l, x1, x2, dy;
@@ -480,32 +480,32 @@ static int iflood(int x, int y, int width, int height,
             labels[y * width + x] = newvalue;
             count++;
         }
-        
+
         if (x >= x1)
             goto skip;
-        
+
         l = x + 1;
-        
+
         if (l < x1)
             PUSH(y, l, x1 - 1, -dy);  /* Leak on left? */
-        
+
         x = x1 + 1;
-        
+
         do {
             for (; x < width && out[y * width + x] != 0 && labels[y * width + x] == oldvalue; x++) {
                 labels[y * width + x] = newvalue;
                 count++;
             }
-            
+
             PUSH(y, l, x - 1, dy);
-            
+
             if (x > x2 + 1)
                 PUSH(y, x2 + 1, x - 1, -dy);  /* Leak on right? */
-            
+
             skip:
-            
+
             for (x++; x <= x2 && !(out[y * width + x] != 0 && labels[y * width + x] == oldvalue); x++);
-            
+
             l = x;
         } while (x <= x2);
     }
@@ -515,11 +515,11 @@ static int iflood(int x, int y, int width, int height,
 /**
  * alg_labeling
  *
- */ 
+ */
 static int alg_labeling(struct context *cnt)
 {
     struct images *imgs = &cnt->imgs;
-    unsigned char *out = imgs->out;
+    unsigned char *out = imgs->img_motion.image_norm;
     int *labels = imgs->labels;
     int ix, iy, pixelpos;
     int width = imgs->width;
@@ -546,17 +546,17 @@ static int alg_labeling(struct context *cnt)
                 labels[pixelpos] = 1;
                 continue;
             }
-            
+
             /* Already visited by iflood */
             if (labels[pixelpos] > 0)
                 continue;
 
             labelsize = iflood(ix, iy, width, height, out, labels, current_label, 0);
-            
+
             if (labelsize > 0) {
-                MOTION_LOG(DBG, TYPE_ALL, NO_ERRNO, "%s: Label: %i (%i) Size: %i (%i,%i)", 
-                            current_label, cnt->current_image->total_labels, 
-                           labelsize, ix, iy);
+                //MOTION_LOG(DBG, TYPE_ALL, NO_ERRNO, "Label: %i (%i) Size: %i (%i,%i)",
+                //            current_label, cnt->current_image->total_labels,
+                //           labelsize, ix, iy);
 
                 /* Label above threshold? Mark it again (add 32768 to labelnumber). */
                 if (labelsize > cnt->threshold) {
@@ -565,12 +565,12 @@ static int alg_labeling(struct context *cnt)
                     imgs->labels_above++;
                 } else if(max_under < labelsize)
                     max_under = labelsize;
-                
+
                 if (imgs->labelsize_max < labelsize) {
                     imgs->labelsize_max = labelsize;
                     imgs->largest_label = current_label;
                 }
-                
+
                 cnt->current_image->total_labels++;
                 current_label++;
             }
@@ -578,10 +578,10 @@ static int alg_labeling(struct context *cnt)
         pixelpos++; /* Compensate for ix < width - 1 */
     }
 
-    MOTION_LOG(DBG, TYPE_ALL, NO_ERRNO, "%s: %i Labels found. Largest connected Area: %i Pixel(s). "
-               "Largest Label: %i", imgs->largest_label, imgs->labelsize_max, 
-               cnt->current_image->total_labels);
-    
+    //MOTION_LOG(DBG, TYPE_ALL, NO_ERRNO, "%i Labels found. Largest connected Area: %i Pixel(s). "
+    //           "Largest Label: %i", imgs->largest_label, imgs->labelsize_max,
+    //           cnt->current_image->total_labels);
+
     /* Return group of significant labels or if that's none, the next largest
      * group (which is under the threshold, but especially for setup gives an
      * idea how close it was).
@@ -589,17 +589,17 @@ static int alg_labeling(struct context *cnt)
     return imgs->labelgroup_max ? imgs->labelgroup_max : max_under;
 }
 
-/** 
- * dilate9 
- *      Dilates a 3x3 box. 
+/**
+ * dilate9
+ *      Dilates a 3x3 box.
  */
 static int dilate9(unsigned char *img, int width, int height, void *buffer)
 {
-    /* 
-     * - row1, row2 and row3 represent lines in the temporary buffer. 
+    /*
+     * - row1, row2 and row3 represent lines in the temporary buffer.
      * - Window is a sliding window containing max values of the columns
      *   in the 3x3 matrix.
-     * - width is an index into the sliding window (this is faster than 
+     * - width is an index into the sliding window (this is faster than
      *   doing modulo 3 on i).
      * - blob keeps the current max value.
      */
@@ -618,7 +618,7 @@ static int dilate9(unsigned char *img, int width, int height, void *buffer)
 
     /* Pointer to the current row in img. */
     yp = img;
-    
+
     for (y = 0; y < height; y++) {
         /* Move down one step; row 1 becomes the previous row 2 and so on. */
         rowTemp = row1;
@@ -631,7 +631,7 @@ static int dilate9(unsigned char *img, int width, int height, void *buffer)
             memset(row3, 0, width);
         else
             memcpy(row3, yp+width, width);
-        
+
         /* Init slots 0 and 1 in the moving window. */
         window[0] = MAX3(row1[0], row2[0], row3[0]);
         window[1] = MAX3(row1[1], row2[1], row3[1]);
@@ -640,7 +640,7 @@ static int dilate9(unsigned char *img, int width, int height, void *buffer)
         blob = MAX2(window[0], window[1]);
         widx = 2;
 
-        /* 
+        /*
          * Iterate over the current row; index i is off by one to eliminate
          * a lot of +1es in the loop.
          */
@@ -648,7 +648,7 @@ static int dilate9(unsigned char *img, int width, int height, void *buffer)
             /* Get the max value of the next column in the 3x3 matrix. */
             latest = window[widx] = MAX3(row1[i], row2[i], row3[i]);
 
-            /* 
+            /*
              * If the value is larger than the current max, use it. Otherwise,
              * calculate a new max (because the new value may not be the max.
              */
@@ -672,33 +672,33 @@ static int dilate9(unsigned char *img, int width, int height, void *buffer)
         *yp = *(yp + width - 1) = 0;
         yp += width;
     }
-    
+
     return sum;
 }
 
-/** 
- * dilate5 
- *      Dilates a + shape. 
+/**
+ * dilate5
+ *      Dilates a + shape.
  */
 static int dilate5(unsigned char *img, int width, int height, void *buffer)
 {
-    /* 
-     * - row1, row2 and row3 represent lines in the temporary buffer. 
+    /*
+     * - row1, row2 and row3 represent lines in the temporary buffer.
      * - mem holds the max value of the overlapping part of two + shapes.
      */
     int y, i, sum = 0;
     unsigned char *row1, *row2, *row3, *rowTemp, *yp;
     unsigned char blob, mem, latest;
-    
+
     /* Set up row pointers in the temporary buffer. */
     row1 = buffer;
     row2 = row1 + width;
     row3 = row2 + width;
-    
+
     /* Init rows 2 and 3. */
     memset(row2, 0, width);
     memcpy(row3, img, width);
-    
+
     /* Pointer to the current row in img. */
     yp = img;
 
@@ -708,7 +708,7 @@ static int dilate5(unsigned char *img, int width, int height, void *buffer)
         row1 = row2;
         row2 = row3;
         row3 = rowTemp;
-        
+
         /* If we're at the last row, fill with zeros, otherwise copy from img. */
         if (y == height - 1)
             memset(row3, 0, width);
@@ -718,11 +718,11 @@ static int dilate5(unsigned char *img, int width, int height, void *buffer)
         /* Init mem and set blob to force an evaluation of the entire + shape. */
         mem = MAX2(row2[0], row2[1]);
         blob = 1; /* dummy value, must be > 0 */
-        
+
         for (i = 1; i < width - 1; i++) {
             /* Get the max value of the "right edge" of the + shape. */
             latest = MAX3(row1[i], row2[i + 1], row3[i]);
-            
+
             if (blob == 0) {
                 /* In case the last blob is zero, only latest matters. */
                 blob = latest;
@@ -747,9 +747,9 @@ static int dilate5(unsigned char *img, int width, int height, void *buffer)
     return sum;
 }
 
-/** 
- * erode9 
- *      Erodes a 3x3 box. 
+/**
+ * erode9
+ *      Erodes a 3x3 box.
  */
 static int erode9(unsigned char *img, int width, int height, void *buffer, unsigned char flag)
 {
@@ -792,8 +792,8 @@ static int erode9(unsigned char *img, int width, int height, void *buffer, unsig
 }
 
 /**
- * erode5 
- *      Erodes in a + shape. 
+ * erode5
+ *      Erodes in a + shape.
  */
 static int erode5(unsigned char *img, int width, int height, void *buffer, unsigned char flag)
 {
@@ -809,7 +809,7 @@ static int erode5(unsigned char *img, int width, int height, void *buffer, unsig
     for (y = 0; y < height; y++) {
         memcpy(Row1, Row2, width);
         memcpy(Row2, Row3, width);
-    
+
         if (y == height-1)
             memset(Row3, flag, width);
         else
@@ -831,14 +831,14 @@ static int erode5(unsigned char *img, int width, int height, void *buffer, unsig
     return sum;
 }
 
-/** 
- * alg_despeckle 
+/**
+ * alg_despeckle
  *      Despeckling routine to remove noisy detections.
  */
 int alg_despeckle(struct context *cnt, int olddiffs)
 {
     int diffs = 0;
-    unsigned char *out = cnt->imgs.out;
+    unsigned char *out = cnt->imgs.img_motion.image_norm;
     int width = cnt->imgs.width;
     int height = cnt->imgs.height;
     int done = 0, i, len = strlen(cnt->conf.despeckle_filter);
@@ -847,12 +847,12 @@ int alg_despeckle(struct context *cnt, int olddiffs)
     for (i = 0; i < len; i++) {
         switch (cnt->conf.despeckle_filter[i]) {
         case 'E':
-            if ((diffs = erode9(out, width, height, common_buffer, 0)) == 0) 
+            if ((diffs = erode9(out, width, height, common_buffer, 0)) == 0)
                 i = len;
             done = 1;
             break;
         case 'e':
-            if ((diffs = erode5(out, width, height, common_buffer, 0)) == 0) 
+            if ((diffs = erode5(out, width, height, common_buffer, 0)) == 0)
                 i = len;
             done = 1;
             break;
@@ -875,18 +875,18 @@ int alg_despeckle(struct context *cnt, int olddiffs)
 
     /* If conf.despeckle_filter contains any valid action EeDdl */
     if (done) {
-        if (done != 2) 
+        if (done != 2)
             cnt->imgs.labelsize_max = 0; // Disable Labeling
         return diffs;
     } else {
         cnt->imgs.labelsize_max = 0; // Disable Labeling
-    }    
-    
+    }
+
     return olddiffs;
 }
 
-/** 
- * alg_tune_smartmask 
+/**
+ * alg_tune_smartmask
  *      Generates actual smartmask. Calculate sensitivity based on motion.
  */
 void alg_tune_smartmask(struct context *cnt)
@@ -919,9 +919,9 @@ void alg_tune_smartmask(struct context *cnt)
             smartmask_final[i] = 255;
     }
     /* Further expansion (here:erode due to inverted logic!) of the mask. */
-    diff = erode9(smartmask_final, cnt->imgs.width, cnt->imgs.height, 
+    diff = erode9(smartmask_final, cnt->imgs.width, cnt->imgs.height,
                   cnt->imgs.common_buffer, 255);
-    diff = erode5(smartmask_final, cnt->imgs.width, cnt->imgs.height, 
+    diff = erode5(smartmask_final, cnt->imgs.width, cnt->imgs.height,
                   cnt->imgs.common_buffer, 255);
 }
 
@@ -939,7 +939,7 @@ int alg_diff_standard(struct context *cnt, unsigned char *new)
     int noise = cnt->noise;
     int smartmask_speed = cnt->smartmask_speed;
     unsigned char *ref = imgs->ref;
-    unsigned char *out = imgs->out;
+    unsigned char *out = imgs->img_motion.image_norm;
     unsigned char *mask = imgs->mask;
     unsigned char *smartmask_final = imgs->smartmask_final;
     int *smartmask_buffer = imgs->smartmask_buffer;
@@ -950,17 +950,17 @@ int alg_diff_standard(struct context *cnt, unsigned char *new)
 
     i = imgs->motionsize;
     memset(out + i, 128, i / 2); /* Motion pictures are now b/w i.o. green */
-    /* 
-     * Keeping this memset in the MMX case when zeroes are necessarily 
+    /*
+     * Keeping this memset in the MMX case when zeroes are necessarily
      * written anyway seems to be beneficial in terms of speed. Perhaps a
      * cache thing?
      */
     memset(out, 0, i);
 
 #ifdef HAVE_MMX
-    /* 
+    /*
      * NOTE: The Pentium has two instruction pipes: U and V. I have grouped MMX
-     * instructions in pairs according to how I think they will be scheduled in 
+     * instructions in pairs according to how I think they will be scheduled in
      * the U and V pipes. Due to pairing constraints, the V pipe will sometimes
      * be empty (for example, memory access always goes into the U pipe).
      *
@@ -972,31 +972,31 @@ int alg_diff_standard(struct context *cnt, unsigned char *new)
      * -- Per Jonsson
      */
 
-    /* 
+    /*
      * To avoid a div, we work with differences multiplied by 255 in the
      * default case and *mask otherwise. Thus, the limit to compare with is
      * 255 * (noise + 1) - 1).
      */
     mmtemp.uw[0] = mmtemp.uw[1] = mmtemp.uw[2] = mmtemp.uw[3] =
                    (unsigned short)(noise * 255 + 254);
-    
-    /* 
+
+    /*
      * Reset mm5 to zero, set the mm6 mask, and store the multiplied noise
      * level as four words in mm7.
      */
     movq_m2r(mmtemp, mm7);             /* U */
     pcmpeqb_r2r(mm6, mm6);             /* V */
-    
+
     pxor_r2r(mm5, mm5);                /* U */
     psrlw_i2r(8, mm6);                 /* V */
 
-    /* 
+    /*
      * We must unload mm5 every 255th round, because the diffs accumulate
      * in each packed byte, which can hold at most 255 diffs before it
      * gets saturated.
      */
     unload = 255;
-    
+
     for (; i > 7; i -= 8) {
         /* Calculate abs(*ref-*new) for 8 pixels in parallel. */
         movq_m2r(*ref, mm0);           /* U: mm0 = r7 r6 r5 r4 r3 r2 r1 r0 */
@@ -1008,14 +1008,14 @@ int alg_diff_standard(struct context *cnt, unsigned char *new)
         /* These subtractions are saturated, i.e. won't go below 0. */
         psubusb_r2r(mm1, mm0);         /* U: mm0 = (r7-n7) ... (r0-n0) */
         psubusb_r2r(mm2, mm1);         /* V: mm1 = (n7-r7) ... (n0-r0) */
-        
+
         /* Each byte dX in mm0 is abs(nX-rX). */
         por_r2r(mm1, mm0);             /* U: mm0 = d7 d6 d5 d4 d3 d2 d1 d0 */
 
         /* Expand the absolute differences to words in mm0 and mm1. */
         movq_r2r(mm0, mm1);            /* U: mm1 = d7 d6 d5 d4 d3 d2 d1 d0 */
         punpcklbw_r2r(mm4, mm0);       /* V: mm0 =    d3    d2    d1    d0 */
-        
+
         punpckhbw_r2r(mm4, mm1);       /* U: mm1 =    d7    d6    d5    d4 */
 
         if (mask) {
@@ -1027,29 +1027,29 @@ int alg_diff_standard(struct context *cnt, unsigned char *new)
 
             movq_r2r(mm2, mm3);        /* U: mm3 = m7 m6 m5 m4 m3 m2 m1 m0 */
             punpcklbw_r2r(mm4, mm2);   /* v: mm2 =    m3    m2    m1    m0 */
-            
+
             punpckhbw_r2r(mm4, mm3);   /* U: mm3 =    m7    m6    m5    m4 */
             pmullw_r2r(mm2, mm0);      /* V: mm0 = (d3*m3) ... (d0*m0) */
-            
+
             pmullw_r2r(mm3, mm1);      /* U: mm1 = (d7*m7) ... (d4*m4) */
 
             mask += 8;
         } else {
-            /* 
+            /*
              * Not using mask - multiply the absolute differences by 255. We
              * do this by left-shifting 8 places and then subtracting dX.
              */
             movq_r2r(mm0, mm2);        /* U: mm2 =    d3    d2    d1    d0 */
-            psllw_i2r(8, mm0);         /* V: mm2 = (256*d3) ... (256*d0) */ 
+            psllw_i2r(8, mm0);         /* V: mm2 = (256*d3) ... (256*d0) */
 
             movq_r2r(mm1, mm3);        /* U: mm3 =    d7    d6    d5    d4 */
             psllw_i2r(8, mm1);         /* V: mm3 = (256*d7) ... (256*d4) */
 
             psubusw_r2r(mm2, mm0);     /* U */
-            psubusw_r2r(mm3, mm1);     /* V */ 
+            psubusw_r2r(mm3, mm1);     /* V */
         }
 
-        /* 
+        /*
          * Next, compare the multiplied absolute differences with the multiplied
          * noise level (repeated as 4 words in mm7), resulting in a "motion flag"
          * for each pixel.
@@ -1057,7 +1057,7 @@ int alg_diff_standard(struct context *cnt, unsigned char *new)
          * Since pcmpgtw performs signed comparisons, we have to subtract noise,
          * test for equality to 0 and then invert the result.
          *
-         * Note that it is safe to generate the "motion flags" before the 
+         * Note that it is safe to generate the "motion flags" before the
          * smartmask code, as all that can happen is that individual flags get
          * reset to 0 because of the smartmask.
          */
@@ -1083,7 +1083,7 @@ int alg_diff_standard(struct context *cnt, unsigned char *new)
              */
             movq_m2r(*smartmask_final, mm3); /* U: mm3 = s7 s6 s5 s4 s3 s2 s1 s0 */
 
-            /* 
+            /*
              * ...but move the "motion flags" to memory before, in order to
              * increment *smartmask_buffer properly below.
              */
@@ -1122,11 +1122,11 @@ int alg_diff_standard(struct context *cnt, unsigned char *new)
 
         /*
          * mm5 holds 8 separate counts - each one is increased according to
-         * the contents of mm4 (where each byte is either 0x00 or 0x01). 
+         * the contents of mm4 (where each byte is either 0x00 or 0x01).
          */
         movq_r2m(mm2, *out);           /* U: this will stall */
         paddusb_r2r(mm4, mm5);         /* V: add counts to mm5 */
-        
+
         /*
          * Every 255th turn, we need to unload mm5 into the diffs variable,
          * because otherwise the packed bytes will get saturated.
@@ -1136,7 +1136,7 @@ int alg_diff_standard(struct context *cnt, unsigned char *new)
             movq_r2m(mm5, mmtemp);     /* U */
             pxor_r2r(mm5, mm5);        /* V: mm5 = 0 */
 
-            diffs += mmtemp.ub[0] + mmtemp.ub[1] + mmtemp.ub[2] + mmtemp.ub[3] + 
+            diffs += mmtemp.ub[0] + mmtemp.ub[1] + mmtemp.ub[2] + mmtemp.ub[3] +
                      mmtemp.ub[4] + mmtemp.ub[5] + mmtemp.ub[6] + mmtemp.ub[7];
             unload = 255;
         }
@@ -1146,13 +1146,13 @@ int alg_diff_standard(struct context *cnt, unsigned char *new)
         new += 8;
     }
 
-    /* 
+    /*
      * Check if there are diffs left in mm5 that need to be copied to the
-     * diffs variable. 
+     * diffs variable.
      */
     if (unload < 255) {
         movq_r2m(mm5, mmtemp);
-        diffs += mmtemp.ub[0] + mmtemp.ub[1] + mmtemp.ub[2] + mmtemp.ub[3] + 
+        diffs += mmtemp.ub[0] + mmtemp.ub[1] + mmtemp.ub[2] + mmtemp.ub[3] +
                  mmtemp.ub[4] + mmtemp.ub[5] + mmtemp.ub[6] + mmtemp.ub[7];
     }
 
@@ -1170,15 +1170,15 @@ int alg_diff_standard(struct context *cnt, unsigned char *new)
         /* Apply fixed mask */
         if (mask)
             curdiff = ((int)(curdiff * *mask++) / 255);
-            
+
         if (smartmask_speed) {
             if (curdiff > noise) {
-                /* 
+                /*
                  * Increase smart_mask sensitivity every frame when motion
                  * is detected. (with speed=5, mask is increased by 1 every
                  * second. To be able to increase by 5 every second (with
                  * speed=10) we add 5 here. NOT related to the 5 at ratio-
-                 * calculation. 
+                 * calculation.
                  */
                 if (cnt->event_nr != cnt->prev_event)
                     (*smartmask_buffer) += SMARTMASK_SENSITIVITY_INCR;
@@ -1202,7 +1202,7 @@ int alg_diff_standard(struct context *cnt, unsigned char *new)
 }
 
 /**
- * alg_diff_fast 
+ * alg_diff_fast
  *      Very fast diff function, does not apply mask overlaying.
  */
 static char alg_diff_fast(struct context *cnt, int max_n_changes, unsigned char *new)
@@ -1233,23 +1233,23 @@ static char alg_diff_fast(struct context *cnt, int max_n_changes, unsigned char
     return 0;
 }
 
-/** 
- * alg_diff 
+/**
+ * alg_diff
  *      Uses diff_fast to quickly decide if there is anything worth
  *      sending to diff_standard.
  */
 int alg_diff(struct context *cnt, unsigned char *new)
 {
     int diffs = 0;
-    
+
     if (alg_diff_fast(cnt, cnt->conf.max_changes / 2, new))
         diffs = alg_diff_standard(cnt, new);
 
     return diffs;
 }
 
-/** 
- * alg_lightswitch 
+/**
+ * alg_lightswitch
  *      Detects a sudden massive change in the picture.
  *      It is assumed to be the light being switched on or a camera displacement.
  *      In any way the user doesn't think it is worth capturing.
@@ -1257,41 +1257,41 @@ int alg_diff(struct context *cnt, unsigned char *new)
 int alg_lightswitch(struct context *cnt, int diffs)
 {
     struct images *imgs = &cnt->imgs;
-    
+
     if (cnt->conf.lightswitch < 0)
         cnt->conf.lightswitch = 0;
     if (cnt->conf.lightswitch > 100)
         cnt->conf.lightswitch = 100;
-    
+
     /* Is lightswitch percent of the image changed? */
     if (diffs > (imgs->motionsize * cnt->conf.lightswitch / 100))
         return 1;
-    
+
     return 0;
 }
 
 /**
  * alg_switchfilter
  *
- */ 
+ */
 int alg_switchfilter(struct context *cnt, int diffs, unsigned char *newimg)
 {
     int linediff = diffs / cnt->imgs.height;
-    unsigned char *out = cnt->imgs.out;
+    unsigned char *out = cnt->imgs.img_motion.image_norm;
     int y, x, line;
     int lines = 0, vertlines = 0;
 
     for (y = 0; y < cnt->imgs.height; y++) {
         line = 0;
         for (x = 0; x < cnt->imgs.width; x++) {
-            if (*(out++)) 
+            if (*(out++))
                 line++;
         }
 
-        if (line > cnt->imgs.width / 18) 
+        if (line > cnt->imgs.width / 18)
             vertlines++;
-        
-        if (line > linediff * 2) 
+
+        if (line > linediff * 2)
             lines++;
     }
 
@@ -1307,13 +1307,13 @@ int alg_switchfilter(struct context *cnt, int diffs, unsigned char *newimg)
     return 0;
 }
 
-/** 
+/**
  * alg_update_reference_frame
  *
  *   Called from 'motion_loop' to calculate the reference frame
  *   Moving objects are excluded from the reference frame for a certain
  *   amount of time to improve detection.
- * 
+ *
  * Parameters:
  *
  *   cnt    - current thread's context struct
@@ -1322,15 +1322,15 @@ int alg_switchfilter(struct context *cnt, int diffs, unsigned char *newimg)
  */
 #define ACCEPT_STATIC_OBJECT_TIME 10  /* Seconds */
 #define EXCLUDE_LEVEL_PERCENT 20
-void alg_update_reference_frame(struct context *cnt, int action) 
+void alg_update_reference_frame(struct context *cnt, int action)
 {
     int accept_timer = cnt->lastrate * ACCEPT_STATIC_OBJECT_TIME;
     int i, threshold_ref;
     int *ref_dyn = cnt->imgs.ref_dyn;
-    unsigned char *image_virgin = cnt->imgs.image_virgin;
+    unsigned char *image_virgin = cnt->imgs.image_virgin.image_norm;
     unsigned char *ref = cnt->imgs.ref;
     unsigned char *smartmask = cnt->imgs.smartmask_final;
-    unsigned char *out = cnt->imgs.out;
+    unsigned char *out = cnt->imgs.img_motion.image_norm;
 
     if (cnt->lastrate > 5)  /* Match rate limit */
         accept_timer /= (cnt->lastrate / 3);
@@ -1367,7 +1367,7 @@ void alg_update_reference_frame(struct context *cnt, int action)
 
     } else {   /* action == RESET_REF_FRAME - also used to initialize the frame at startup. */
         /* Copy fresh image */
-        memcpy(cnt->imgs.ref, cnt->imgs.image_virgin, cnt->imgs.size);
+        memcpy(cnt->imgs.ref, cnt->imgs.image_virgin.image_norm, cnt->imgs.size_norm);
         /* Reset static objects */
         memset(cnt->imgs.ref_dyn, 0, cnt->imgs.motionsize * sizeof(*cnt->imgs.ref_dyn));
     }
