@@ -30,17 +30,15 @@ typedef enum {
     EVENT_CRITICAL,
     EVENT_AREA_DETECTED,
     EVENT_CAMERA_LOST,
+    EVENT_CAMERA_FOUND,
     EVENT_FFMPEG_PUT,
-    EVENT_SDL_PUT,
     EVENT_LAST,
 } motion_event;
 
+typedef void(* event_handler)(struct context *, motion_event, struct image_data *,
+             char *, void *, struct timeval *);
 
-typedef void(* event_handler)(struct context *, motion_event, unsigned char *,
-             char *, void *, struct tm *);
-
-void event(struct context *, motion_event, unsigned char *, char *, void *,
-           struct tm *);
+void event(struct context *, motion_event, struct image_data *img_data, char *, void *, struct timeval *);
 const char * imageext(struct context *);
 
 #endif /* _INCLUDE_EVENT_H_ */
