@@ -14,7 +14,11 @@
 
 #include <assert.h>
 
-#undef HAVE_STDLIB_H
+#ifdef HAVE_WEBP
+#include <webp/encode.h>
+#include <webp/mux.h>
+#endif /* HAVE_WEBP */
+
 #include <jpeglib.h>
 #include <jerror.h>
 
@@ -171,21 +175,21 @@ struct tiff_writing {
 static void put_direntry(struct tiff_writing *into, const char *data, unsigned length)
 {
     if (length <= 4) {
-	    /* Entries that fit in the directory entry are stored there */
-	    memset(into->buf, 0, 4);
-	    memcpy(into->buf, data, length);
+        /* Entries that fit in the directory entry are stored there */
+        memset(into->buf, 0, 4);
+        memcpy(into->buf, data, length);
     } else {
-	    /* Longer entries are stored out-of-line */
-	    unsigned offset = into->data_offset;
+        /* Longer entries are stored out-of-line */
+        unsigned offset = into->data_offset;
 
         while ((offset & 0x03) != 0) {  /* Alignment */
-	        into->base[offset] = 0;
-	        offset ++;
-	    }
+            into->base[offset] = 0;
+            offset ++;
+        }
 
-	    put_uint32(into->buf, offset);
-	    memcpy(into->base + offset, data, length);
-	    into->data_offset = offset + length;
+        put_uint32(into->buf, offset);
+        memcpy(into->base + offset, data, length);
+        into->data_offset = offset + length;
     }
 }
 
@@ -222,28 +226,30 @@ static void put_subjectarea(struct tiff_writing *into, const struct coord *box)
  * any image data is written by jpeg_write_scanlines().
  */
 static void put_jpeg_exif(j_compress_ptr cinfo,
-			  const struct context *cnt,
-			  const struct tm *timestamp,
-			  const struct coord *box)
+              const struct context *cnt,
+              const struct timeval *tv1,
+              const struct coord *box)
 {
     /* description, datetime, and subtime are the values that are actually
      * put into the EXIF data
     */
     char *description, *datetime, *subtime;
     char datetime_buf[22];
+    struct tm timestamp_tm;
 
-    if (timestamp) {
-	/* Exif requires this exact format */
-	    snprintf(datetime_buf, 21, "%04d:%02d:%02d %02d:%02d:%02d",
-		        timestamp->tm_year + 1900,
-		        timestamp->tm_mon + 1,
-		        timestamp->tm_mday,
-		        timestamp->tm_hour,
-		        timestamp->tm_min,
-		        timestamp->tm_sec);
-	    datetime = datetime_buf;
+    if (tv1->tv_sec) {
+        localtime_r(&tv1->tv_sec, &timestamp_tm);
+    /* Exif requires this exact format */
+        snprintf(datetime_buf, 21, "%04d:%02d:%02d %02d:%02d:%02d",
+                timestamp_tm.tm_year + 1900,
+                timestamp_tm.tm_mon + 1,
+                timestamp_tm.tm_mday,
+                timestamp_tm.tm_hour,
+                timestamp_tm.tm_min,
+                timestamp_tm.tm_sec);
+        datetime = datetime_buf;
     } else {
-	    datetime = NULL;
+        datetime = NULL;
     }
 
     // TODO: Extract subsecond timestamp from somewhere, but only
@@ -251,12 +257,10 @@ static void put_jpeg_exif(j_compress_ptr cinfo,
     subtime = NULL;
 
     if (cnt->conf.exif_text) {
-	    description = malloc(PATH_MAX);
-	    mystrftime(cnt, description, PATH_MAX-1,
-		        cnt->conf.exif_text,
-		        timestamp, NULL, 0);
+        description = malloc(PATH_MAX);
+        mystrftime(cnt, description, PATH_MAX-1, cnt->conf.exif_text, tv1, NULL, 0);
     } else {
-	    description = NULL;
+        description = NULL;
     }
 
     /* Calculate an upper bound on the size of the APP1 marker so
@@ -269,52 +273,52 @@ static void put_jpeg_exif(j_compress_ptr cinfo,
     unsigned datasize = 0;
 
     if (description) {
-	    ifd0_tagcount ++;
-	    datasize += 5 + strlen(description); /* Add 5 for NUL and alignment */
+        ifd0_tagcount ++;
+        datasize += 5 + strlen(description); /* Add 5 for NUL and alignment */
     }
 
     if (datetime) {
-	/* We write this to both the TIFF datetime tag (which most programs
-	 * treat as "last-modified-date") and the EXIF "time of creation of
-	 * original image" tag (which many programs ignore). This is
-	 * redundant but seems to be the thing to do.
-	 */
-	    ifd0_tagcount++;
-	    ifd1_tagcount++;
-	    /* We also write the timezone-offset tag in IFD0 */
-	    ifd0_tagcount++;
-	    /* It would be nice to use the same offset for both tags' values,
-	    * but I don't want to write the bookkeeping for that right now */
-	    datasize += 2 * (5 + strlen(datetime));
+    /* We write this to both the TIFF datetime tag (which most programs
+     * treat as "last-modified-date") and the EXIF "time of creation of
+     * original image" tag (which many programs ignore). This is
+     * redundant but seems to be the thing to do.
+     */
+        ifd0_tagcount++;
+        ifd1_tagcount++;
+        /* We also write the timezone-offset tag in IFD0 */
+        ifd0_tagcount++;
+        /* It would be nice to use the same offset for both tags' values,
+        * but I don't want to write the bookkeeping for that right now */
+        datasize += 2 * (5 + strlen(datetime));
     }
 
     if (subtime) {
-	    ifd1_tagcount++;
-	    datasize += 5 + strlen(subtime);
+        ifd1_tagcount++;
+        datasize += 5 + strlen(subtime);
     }
 
     if (box) {
-	    ifd1_tagcount++;
-	    datasize += 2 * 4;  /* Four 16-bit ints */
+        ifd1_tagcount++;
+        datasize += 2 * 4;  /* Four 16-bit ints */
     }
 
     if (ifd1_tagcount > 0) {
-	    /* If we're writing the Exif sub-IFD, account for the
-	    * two tags that requires */
-	    ifd0_tagcount ++; /* The tag in IFD0 that points to IFD1 */
-	    ifd1_tagcount ++; /* The EXIF version tag */
+        /* If we're writing the Exif sub-IFD, account for the
+        * two tags that requires */
+        ifd0_tagcount ++; /* The tag in IFD0 that points to IFD1 */
+        ifd1_tagcount ++; /* The EXIF version tag */
     }
 
     /* Each IFD takes 12 bytes per tag, plus six more (the tag count and the
      * pointer to the next IFD, always zero in our case)
      */
-    unsigned int ifds_size =
-	( ifd1_tagcount > 0 ? ( 12 * ifd1_tagcount + 6 ) : 0 ) +
-	( ifd0_tagcount > 0 ? ( 12 * ifd0_tagcount + 6 ) : 0 );
+    int ifds_size =
+    ( ifd1_tagcount > 0 ? ( 12 * ifd1_tagcount + 6 ) : 0 ) +
+    ( ifd0_tagcount > 0 ? ( 12 * ifd0_tagcount + 6 ) : 0 );
 
     if (ifds_size == 0) {
-	    /* We're not actually going to write any information. */
-	    return;
+        /* We're not actually going to write any information. */
+        return;
     }
 
     unsigned int buffer_size = 6 /* EXIF marker signature */ +
@@ -326,9 +330,9 @@ static void put_jpeg_exif(j_compress_ptr cinfo,
     memcpy(marker, exif_marker_start, 14); /* EXIF and TIFF headers */
 
     struct tiff_writing writing = (struct tiff_writing) {
-	.base = marker + 6, /* base address for intra-TIFF offsets */
-	.buf = marker + 14, /* current write position */
-	.data_offset = 8 + ifds_size, /* where to start storing data */
+    .base = marker + 6, /* base address for intra-TIFF offsets */
+    .buf = marker + 14, /* current write position */
+    .data_offset = 8 + ifds_size, /* where to start storing data */
     };
 
     /* Write IFD 0 */
@@ -337,22 +341,22 @@ static void put_jpeg_exif(j_compress_ptr cinfo,
     writing.buf += 2;
 
     if (description)
-	    put_stringentry(&writing, TIFF_TAG_IMAGE_DESCRIPTION, description, 0);
+        put_stringentry(&writing, TIFF_TAG_IMAGE_DESCRIPTION, description, 0);
 
     if (datetime)
-	    put_stringentry(&writing, TIFF_TAG_DATETIME, datetime, 1);
+        put_stringentry(&writing, TIFF_TAG_DATETIME, datetime, 1);
 
     if (ifd1_tagcount > 0) {
-	    /* Offset of IFD1 - TIFF header + IFD0 size. */
-	    unsigned ifd1_offset = 8 + 6 + ( 12 * ifd0_tagcount );
-	    memcpy(writing.buf, exif_subifd_tag, 8);
-	    put_uint32(writing.buf + 8, ifd1_offset);
-	    writing.buf += 12;
+        /* Offset of IFD1 - TIFF header + IFD0 size. */
+        unsigned ifd1_offset = 8 + 6 + ( 12 * ifd0_tagcount );
+        memcpy(writing.buf, exif_subifd_tag, 8);
+        put_uint32(writing.buf + 8, ifd1_offset);
+        writing.buf += 12;
     }
 
     if (datetime) {
         memcpy(writing.buf, exif_tzoffset_tag, 12);
-        put_sint16(writing.buf+8, timestamp->tm_gmtoff / 3600);
+        put_sint16(writing.buf+8, timestamp_tm.tm_gmtoff / 3600);
         writing.buf += 12;
     }
 
@@ -361,23 +365,23 @@ static void put_jpeg_exif(j_compress_ptr cinfo,
 
     /* Write IFD 1 */
     if (ifd1_tagcount > 0) {
-	    /* (remember that the tags in any IFD must be in numerical order
-	    * by tag) */
-	    put_uint16(writing.buf, ifd1_tagcount);
-	    memcpy(writing.buf + 2, exif_version_tag, 12); /* tag 0x9000 */
-	    writing.buf += 14;
+        /* (remember that the tags in any IFD must be in numerical order
+        * by tag) */
+        put_uint16(writing.buf, ifd1_tagcount);
+        memcpy(writing.buf + 2, exif_version_tag, 12); /* tag 0x9000 */
+        writing.buf += 14;
 
-	    if (datetime)
-	        put_stringentry(&writing, EXIF_TAG_ORIGINAL_DATETIME, datetime, 1);
+        if (datetime)
+            put_stringentry(&writing, EXIF_TAG_ORIGINAL_DATETIME, datetime, 1);
 
         if (box)
-	        put_subjectarea(&writing, box);
+            put_subjectarea(&writing, box);
 
         if (subtime)
-	        put_stringentry(&writing, EXIF_TAG_ORIGINAL_DATETIME_SS, subtime, 0);
+            put_stringentry(&writing, EXIF_TAG_ORIGINAL_DATETIME_SS, subtime, 0);
 
-	    put_uint32(writing.buf, 0); /* Next IFD = 0 (no next IFD) */
-	    writing.buf += 4;
+        put_uint32(writing.buf, 0); /* Next IFD = 0 (no next IFD) */
+        writing.buf += 4;
     }
 
     /* We should have met up with the OOL data */
@@ -413,8 +417,8 @@ static void put_jpeg_exif(j_compress_ptr cinfo,
  * Returns buffer size of jpeg image
  */
 static int put_jpeg_yuv420p_memory(unsigned char *dest_image, int image_size,
-				   unsigned char *input_image, int width, int height, int quality,
-				   struct context *cnt, struct tm *tm, struct coord *box)
+                   unsigned char *input_image, int width, int height, int quality,
+                   struct context *cnt, struct timeval *tv1, struct coord *box)
 
 {
     int i, j, jpeg_image_size;
@@ -452,14 +456,14 @@ static int put_jpeg_yuv420p_memory(unsigned char *dest_image, int image_size,
 
     jpeg_set_quality(&cinfo, quality, TRUE);
     cinfo.dct_method = JDCT_FASTEST;
-    
+
     _jpeg_mem_dest(&cinfo, dest_image, image_size);  // Data written to mem
-    
+
 
     jpeg_start_compress(&cinfo, TRUE);
 
-    put_jpeg_exif(&cinfo, cnt, tm, box);
-    
+    put_jpeg_exif(&cinfo, cnt, tv1, box);
+
     /* If the image is not a multiple of 16, this overruns the buffers
      * we'll just pad those last bytes with zeros
      */
@@ -469,13 +473,13 @@ static int put_jpeg_yuv420p_memory(unsigned char *dest_image, int image_size,
                 y[i] = input_image + width * (i + j);
                 if (i % 2 == 0) {
                     cb[i / 2] = input_image + width * height + width / 2 * ((i + j) /2);
-                    cr[i / 2] = input_image + width * height + width * height / 4 + width / 2 * ((i + j) / 2);                
+                    cr[i / 2] = input_image + width * height + width * height / 4 + width / 2 * ((i + j) / 2);
                 }
             } else {
                 y[i] = 0x00;
                 cb[i] = 0x00;
                 cr[i] = 0x00;
-            }    
+            }
         }
         jpeg_write_raw_data(&cinfo, data, 16);
     }
@@ -540,6 +544,80 @@ static int put_jpeg_grey_memory(unsigned char *dest_image, int image_size, unsig
     return dest_image_size;
 }
 
+#ifdef HAVE_WEBP
+/**
+ * put_webp_yuv420p_file
+ *      Converts an YUV420P coded image to a webp image and writes
+ *      it to an already open file.
+ *
+ * Inputs:
+ * - image is the image in YUV420P format.
+ * - width and height are the dimensions of the image
+ * - quality is the webp encoding quality 0-100%
+ *
+ * Output:
+ * - The webp is written directly to the file given by the file pointer fp
+ *
+ * Returns nothing
+ */
+static void put_webp_yuv420p_file(FILE *fp,
+                  unsigned char *image, int width, int height,
+                  int quality)
+{
+    /* Create a config present and check for compatible library version */
+    WebPConfig webp_config;
+    if (!WebPConfigPreset(&webp_config, WEBP_PRESET_DEFAULT, (float) quality)){
+        MOTION_LOG(ERR, TYPE_CORE, NO_ERRNO, "libwebp version error");
+        return;
+    }
+
+    /* Create the input data structure and check for compatible library version */
+    WebPPicture webp_image;
+    if (!WebPPictureInit(&webp_image)){
+        MOTION_LOG(ERR, TYPE_CORE, NO_ERRNO, "libwebp version error");
+        return;
+    }
+
+    /* Allocate the image buffer based on image width and height */
+    webp_image.width = width;
+    webp_image.height = height;
+    if (!WebPPictureAlloc(&webp_image)){
+        MOTION_LOG(ERR, TYPE_CORE, NO_ERRNO, "libwebp image buffer allocation error");
+        return;
+    }
+
+    /* Map the input YUV420P buffer as individual Y, U and V pointers */
+    webp_image.y = image;
+    webp_image.u = image + width * height;
+    webp_image.v = webp_image.u + (width * height) / 4;
+
+    /* Setup the memory writting method */
+    WebPMemoryWriter webp_writer;
+    WebPMemoryWriterInit(&webp_writer);
+    webp_image.writer = WebPMemoryWrite;
+    webp_image.custom_ptr = (void*) &webp_writer;
+
+    /* Encode the YUV image as webp */
+    if (!WebPEncode(&webp_config, &webp_image))
+        MOTION_LOG(WRN, TYPE_CORE, NO_ERRNO, "libwebp image compression error");
+
+    /* Write the webp bytestream to file */
+    if (fwrite(webp_writer.mem, sizeof(uint8_t), webp_writer.size, fp) != webp_writer.size)
+        MOTION_LOG(ERR, TYPE_CORE, NO_ERRNO, "unable to save webp image to file");
+
+#if WEBP_ENCODER_ABI_VERSION > 0x0202
+    /* writer.mem must be freed by calling WebPMemoryWriterClear */
+    WebPMemoryWriterClear(&webp_writer);
+#else
+    /* writer.mem must be freed by calling 'free(writer.mem)' */
+    free(webp_writer.mem);
+#endif /* WEBP_ENCODER_ABI_VERSION */
+
+    /* free the memory used by webp for image object */
+    WebPPictureFree(&webp_image);
+}
+#endif /* HAVE_WEBP */
+
 /**
  * put_jpeg_yuv420p_file
  *      Converts an YUV420P coded image to a jpeg image and writes
@@ -556,9 +634,9 @@ static int put_jpeg_grey_memory(unsigned char *dest_image, int image_size, unsig
  * Returns nothing
  */
 static void put_jpeg_yuv420p_file(FILE *fp,
-				  unsigned char *image, int width, int height,
-				  int quality,
-				  struct context *cnt, struct tm *tm, struct coord *box)
+                  unsigned char *image, int width, int height,
+                  int quality,
+                  struct context *cnt, struct timeval *tv1, struct coord *box)
 {
     int i, j;
 
@@ -599,7 +677,7 @@ static void put_jpeg_yuv420p_file(FILE *fp,
     jpeg_stdio_dest(&cinfo, fp);        // Data written to file
     jpeg_start_compress(&cinfo, TRUE);
 
-    put_jpeg_exif(&cinfo, cnt, tm, box);
+    put_jpeg_exif(&cinfo, cnt, tv1, box);
 
     for (j = 0; j < height; j += 16) {
         for (i = 0; i < 16; i++) {
@@ -613,8 +691,8 @@ static void put_jpeg_yuv420p_file(FILE *fp,
                 y[i] = 0x00;
                 cb[i] = 0x00;
                 cr[i] = 0x00;
-            }        
-        }    
+            }
+        }
         jpeg_write_raw_data(&cinfo, data, 16);
     }
 
@@ -892,39 +970,54 @@ void overlay_largest_label(struct context *cnt, unsigned char *out)
  *
  * Returns the dest_image_size if successful. Otherwise 0.
  */
-int put_picture_memory(struct context *cnt, unsigned char* dest_image, int image_size,
-                       unsigned char *image, int quality)
+int put_picture_memory(struct context *cnt, unsigned char* dest_image, int image_size, unsigned char *image,
+        int quality, int width, int height)
 {
     switch (cnt->imgs.type) {
     case VIDEO_PALETTE_YUV420P:
         return put_jpeg_yuv420p_memory(dest_image, image_size, image,
-                                       cnt->imgs.width, cnt->imgs.height, quality, cnt, &(cnt->current_image->timestamp_tm), &(cnt->current_image->location));
+                                       width, height, quality, cnt
+                                       , &(cnt->current_image->timestamp_tv)
+                                       , &(cnt->current_image->location));
     case VIDEO_PALETTE_GREY:
-        return put_jpeg_grey_memory(dest_image, image_size, image,
-                                    cnt->imgs.width, cnt->imgs.height, quality);
+        return put_jpeg_grey_memory(dest_image, image_size, image, width, height, quality);
     default:
-        MOTION_LOG(WRN, TYPE_ALL, NO_ERRNO, "%s: Unknown image type %d",
-                   cnt->imgs.type);
+        MOTION_LOG(WRN, TYPE_ALL, NO_ERRNO, "Unknown image type %d", cnt->imgs.type);
     }
 
     return 0;
 }
 
-void put_picture_fd(struct context *cnt, FILE *picture, unsigned char *image, int quality)
-{
+static void put_picture_fd(struct context *cnt, FILE *picture, unsigned char *image, int quality, int ftype){
+    int width, height;
+    int passthrough;
+
+    passthrough = util_check_passthrough(cnt);
+    if ((ftype == FTYPE_IMAGE) && (cnt->imgs.size_high > 0) && (!passthrough)) {
+        width = cnt->imgs.width_high;
+        height = cnt->imgs.height_high;
+    } else {
+        width = cnt->imgs.width;
+        height = cnt->imgs.height;
+    }
+
     if (cnt->imgs.picture_type == IMAGE_TYPE_PPM) {
-        put_ppm_bgr24_file(picture, image, cnt->imgs.width, cnt->imgs.height);
+        put_ppm_bgr24_file(picture, image, width, height);
     } else {
         switch (cnt->imgs.type) {
         case VIDEO_PALETTE_YUV420P:
-            put_jpeg_yuv420p_file(picture, image, cnt->imgs.width, cnt->imgs.height, quality, cnt, &(cnt->current_image->timestamp_tm), &(cnt->current_image->location));
+            #ifdef HAVE_WEBP
+            if (cnt->imgs.picture_type == IMAGE_TYPE_WEBP)
+                put_webp_yuv420p_file(picture, image, width, height, quality);
+            #endif /* HAVE_WEBP */
+            if (cnt->imgs.picture_type == IMAGE_TYPE_JPEG)
+                put_jpeg_yuv420p_file(picture, image, width, height, quality, cnt, &(cnt->current_image->timestamp_tv), &(cnt->current_image->location));
             break;
         case VIDEO_PALETTE_GREY:
-            put_jpeg_grey_file(picture, image, cnt->imgs.width, cnt->imgs.height, quality);
+            put_jpeg_grey_file(picture, image, width, height, quality);
             break;
         default:
-            MOTION_LOG(WRN, TYPE_ALL, NO_ERRNO, "%s: Unknown image type %d",
-                       cnt->imgs.type);
+            MOTION_LOG(WRN, TYPE_ALL, NO_ERRNO, "Unknown image type %d", cnt->imgs.type);
         }
     }
 }
@@ -939,19 +1032,20 @@ void put_picture(struct context *cnt, char *file, unsigned char *image, int ftyp
         /* Report to syslog - suggest solution if the problem is access rights to target dir. */
         if (errno ==  EACCES) {
             MOTION_LOG(ERR, TYPE_ALL, SHOW_ERRNO,
-                       "%s: Can't write picture to file %s - check access rights to target directory\n"
+                       "Can't write picture to file %s - check access rights to target directory\n"
                        "Thread is going to finish due to this fatal error", file);
             cnt->finish = 1;
             cnt->restart = 0;
             return;
         } else {
             /* If target dir is temporarily unavailable we may survive. */
-            MOTION_LOG(ERR, TYPE_ALL, SHOW_ERRNO, "%s: Can't write picture to file %s", file);
+            MOTION_LOG(ERR, TYPE_ALL, SHOW_ERRNO, "Can't write picture to file %s", file);
             return;
         }
     }
 
-    put_picture_fd(cnt, picture, image, cnt->conf.quality);
+    put_picture_fd(cnt, picture, image, cnt->conf.quality, ftype);
+
     myfclose(picture);
     event(cnt, EVENT_FILECREATE, NULL, file, (void *)(unsigned long)ftype, NULL);
 }
@@ -970,12 +1064,12 @@ unsigned char *get_pgm(FILE *picture, int width, int height)
     line[255] = 0;
 
     if (!fgets(line, 255, picture)) {
-        MOTION_LOG(ERR, TYPE_ALL, SHOW_ERRNO, "%s: Could not read from ppm file");
+        MOTION_LOG(ERR, TYPE_ALL, SHOW_ERRNO, "Could not read from pgm file");
         return NULL;
     }
 
     if (strncmp(line, "P5", 2)) {
-        MOTION_LOG(ERR, TYPE_ALL, SHOW_ERRNO, "%s: This is not a ppm file, starts with '%s'",
+        MOTION_LOG(ERR, TYPE_ALL, SHOW_ERRNO, "This is not a pgm file, starts with '%s'",
                    line);
         return NULL;
     }
@@ -988,7 +1082,7 @@ unsigned char *get_pgm(FILE *picture, int width, int height)
 
     /* Read image size */
     if (sscanf(line, "%d %d", &mask_width, &mask_height) != 2) {
-        MOTION_LOG(ERR, TYPE_ALL, SHOW_ERRNO, "%s: Failed reading size in pgm file");
+        MOTION_LOG(ERR, TYPE_ALL, SHOW_ERRNO, "Failed reading size in pgm file");
         return NULL;
     }
 
@@ -999,17 +1093,20 @@ unsigned char *get_pgm(FILE *picture, int width, int height)
             return NULL;
 
     if (sscanf(line, "%d", &maxval) != 1) {
-        MOTION_LOG(ERR, TYPE_ALL, SHOW_ERRNO, "%s: Failed reading maximum value in pgm file");
+        MOTION_LOG(ERR, TYPE_ALL, SHOW_ERRNO, "Failed reading maximum value in pgm file");
         return NULL;
     }
 
     /* Read data */
-
-    image = mymalloc(mask_width * mask_height);
+    /* We allocate the size for a 420P since we will use
+    ** this image for masking privacy which needs the space for
+    ** the cr / cb components
+    */
+    image = mymalloc((mask_width * mask_height * 3) / 2);
 
     for (y = 0; y < mask_height; y++) {
         if ((int)fread(&image[y * mask_width], 1, mask_width, picture) != mask_width)
-            MOTION_LOG(ERR, TYPE_ALL, SHOW_ERRNO, "%s: Failed reading image data from pgm file");
+            MOTION_LOG(ERR, TYPE_ALL, SHOW_ERRNO, "Failed reading image data from pgm file");
 
         for (x = 0; x < mask_width; x++)
             image[y * mask_width + x] = (int)image[y * mask_width + x] * 255 / maxval;
@@ -1018,16 +1115,16 @@ unsigned char *get_pgm(FILE *picture, int width, int height)
 
     /* Resize mask if required */
     if (mask_width != width || mask_height != height) {
-        MOTION_LOG(WRN, TYPE_ALL, NO_ERRNO, "%s: The mask file specified is not the same size as image from camera.");
-        MOTION_LOG(WRN, TYPE_ALL, NO_ERRNO, "%s: Attempting to resize mask image from %dx%d to %dx%d",
+        MOTION_LOG(WRN, TYPE_ALL, NO_ERRNO, "The mask file specified is not the same size as image from camera.");
+        MOTION_LOG(WRN, TYPE_ALL, NO_ERRNO, "Attempting to resize mask image from %dx%d to %dx%d",
                    mask_width, mask_height, width, height);
 
-        resized_image = mymalloc(width * height);
+        resized_image = mymalloc((width * height * 3) / 2);
 
         for (y = 0; y < height; y++) {
             for (x = 0; x < width; x++) {
                 resized_image[y * width + x] = image[
-                        (mask_height - 1) * y / (height - 1) * mask_width + 
+                        (mask_height - 1) * y / (height - 1) * mask_width +
                         (mask_width  - 1) * x / (width  - 1)];
             }
         }
@@ -1056,15 +1153,15 @@ void put_fixed_mask(struct context *cnt, const char *file)
         /* Report to syslog - suggest solution if the problem is access rights to target dir. */
         if (errno ==  EACCES) {
             MOTION_LOG(ERR, TYPE_ALL, SHOW_ERRNO,
-                       "%s: can't write mask file %s - check access rights to target directory",
+                       "can't write mask file %s - check access rights to target directory",
                        file);
         } else {
             /* If target dir is temporarily unavailable we may survive. */
-            MOTION_LOG(ERR, TYPE_ALL, SHOW_ERRNO, "%s: can't write mask file %s", file);
+            MOTION_LOG(ERR, TYPE_ALL, SHOW_ERRNO, "can't write mask file %s", file);
         }
         return;
     }
-    memset(cnt->imgs.out, 255, cnt->imgs.motionsize); /* Initialize to unset */
+    memset(cnt->imgs.img_motion.image_norm, 255, cnt->imgs.motionsize); /* Initialize to unset */
 
     /* Write pgm-header. */
     fprintf(picture, "P5\n");
@@ -1072,14 +1169,14 @@ void put_fixed_mask(struct context *cnt, const char *file)
     fprintf(picture, "%d\n", 255);
 
     /* Write pgm image data at once. */
-    if ((int)fwrite(cnt->imgs.out, cnt->conf.width, cnt->conf.height, picture) != cnt->conf.height) {
-        MOTION_LOG(ERR, TYPE_ALL, SHOW_ERRNO, "%s: Failed writing default mask as pgm file");
+    if ((int)fwrite(cnt->imgs.img_motion.image_norm, cnt->conf.width, cnt->conf.height, picture) != cnt->conf.height) {
+        MOTION_LOG(ERR, TYPE_ALL, SHOW_ERRNO, "Failed writing default mask as pgm file");
         return;
     }
 
     myfclose(picture);
 
-    MOTION_LOG(ERR, TYPE_ALL, NO_ERRNO, "%s: Creating empty mask %s\nPlease edit this file and "
+    MOTION_LOG(ERR, TYPE_ALL, NO_ERRNO, "Creating empty mask %s\nPlease edit this file and "
                "re-run motion to enable mask feature", cnt->conf.mask_file);
 }
 
@@ -1097,6 +1194,7 @@ void preview_save(struct context *cnt)
     char previewname[PATH_MAX];
     char filename[PATH_MAX];
     struct image_data *saved_current_image;
+    int passthrough;
 
     if (cnt->imgs.preview_image.diffs) {
         /* Save current global context. */
@@ -1107,12 +1205,7 @@ void preview_save(struct context *cnt)
         /* Use filename of movie i.o. jpeg_filename when set to 'preview'. */
         use_imagepath = strcmp(cnt->conf.imagepath, "preview");
 
-#ifdef HAVE_FFMPEG
-        if ((cnt->ffmpeg_output || (cnt->conf.useextpipe && cnt->extpipe))
-            && !use_imagepath) {
-#else
-        if ((cnt->conf.useextpipe && cnt->extpipe) && !use_imagepath) {
-#endif
+        if ((cnt->ffmpeg_output || (cnt->conf.useextpipe && cnt->extpipe)) && !use_imagepath) {
             if (cnt->conf.useextpipe && cnt->extpipe) {
                 basename_len = strlen(cnt->extpipefilename) + 1;
                 strncpy(previewname, cnt->extpipefilename, basename_len);
@@ -1125,13 +1218,20 @@ void preview_save(struct context *cnt)
 
             previewname[basename_len] = '\0';
             strcat(previewname, imageext(cnt));
-            put_picture(cnt, previewname, cnt->imgs.preview_image.image , FTYPE_IMAGE);
+
+            passthrough = util_check_passthrough(cnt);
+            if ((cnt->imgs.size_high > 0) && (!passthrough)) {
+                put_picture(cnt, previewname, cnt->imgs.preview_image.image_high , FTYPE_IMAGE);
+            } else {
+                put_picture(cnt, previewname, cnt->imgs.preview_image.image_norm , FTYPE_IMAGE);
+            }
+
         } else {
             /*
              * Save best preview-shot also when no movies are recorded or imagepath
              * is used. Filename has to be generated - nothing available to reuse!
              */
-            MOTION_LOG(NTC, TYPE_ALL, NO_ERRNO, "%s: different filename or picture only!");
+            MOTION_LOG(NTC, TYPE_ALL, NO_ERRNO, "different filename or picture only!");
             /*
              * conf.imagepath would normally be defined but if someone deleted it by
              * control interface it is better to revert to the default than fail.
@@ -1141,13 +1241,46 @@ void preview_save(struct context *cnt)
             else
                 imagepath = (char *)DEF_IMAGEPATH;
 
-            mystrftime(cnt, filename, sizeof(filename), imagepath, &cnt->imgs.preview_image.timestamp_tm, NULL, 0);
+            mystrftime(cnt, filename, sizeof(filename), imagepath, &cnt->imgs.preview_image.timestamp_tv, NULL, 0);
             snprintf(previewname, PATH_MAX, "%s/%s.%s", cnt->conf.filepath, filename, imageext(cnt));
 
-            put_picture(cnt, previewname, cnt->imgs.preview_image.image, FTYPE_IMAGE);
+            passthrough = util_check_passthrough(cnt);
+            if ((cnt->imgs.size_high > 0) && (!passthrough)) {
+                put_picture(cnt, previewname, cnt->imgs.preview_image.image_high , FTYPE_IMAGE);
+            } else {
+                put_picture(cnt, previewname, cnt->imgs.preview_image.image_norm, FTYPE_IMAGE);
+            }
         }
 
         /* Restore global context values. */
         cnt->current_image = saved_current_image;
     }
 }
+
+/**
+ * scale_half_yuv420p
+ *      scale down by half yuv420p
+ *
+ * Returns pointer to scaled image
+ */
+
+unsigned char *scale_half_yuv420p(int origwidth, int origheight, unsigned char *img)
+{
+    /* allocate buffer for resized image */
+    unsigned char *scaled_img = mymalloc ((origwidth/2 * origheight/2) * 3 / 2);
+
+    int i = 0, x, y;
+    for (y = 0; y < origheight; y+=2)
+        for (x = 0; x < origwidth; x+=2)
+            scaled_img[i++] = img[y * origwidth + x];
+
+    for (y = 0; y < origheight / 2; y+=2)
+       for (x = 0; x < origwidth; x += 4)
+       {
+          scaled_img[i++] = img[(origwidth * origheight) + (y * origwidth) + x];
+          scaled_img[i++] = img[(origwidth * origheight) + (y * origwidth) + (x + 1)];
+       }
+
+    return scaled_img;
+}
+
