@@ -25,7 +25,7 @@
 #include <arpa/inet.h>
 #include <netdb.h>
 #include <ctype.h>
-#include <sys/fcntl.h>
+#include <fcntl.h>
 
 #define STREAM_REALM       "Motion Stream Security Access"
 #define KEEP_ALIVE_TIMEOUT 100
@@ -33,12 +33,35 @@
 typedef void* (*auth_handler)(void*);
 struct auth_param {
     struct context *cnt;
+    struct stream *stm;
+    int *stream_count;
     int sock;
     int sock_flags;
     int* thread_count;
     struct config *conf;
 };
 
+/**
+ * get_host
+ *      Gets the host (IP) of a client from the socket file descriptor
+ * Returns nothing
+ */
+static void get_host(char *buf, int fd)
+{
+    struct sockaddr_storage client;
+    socklen_t client_len = sizeof(client);
+    int res = getpeername(fd, (struct sockaddr *)&client, &client_len);
+    if (res != 0)
+        return;
+
+    char host[NI_MAXHOST];
+    res = getnameinfo((struct sockaddr *)&client, client_len, host, sizeof(host), NULL, 0, NI_NUMERICHOST);
+    if (res != 0)
+        return;
+
+    strncpy(buf, host, NI_MAXHOST - 1);
+}
+
 pthread_mutex_t stream_auth_mutex;
 
 /**
@@ -54,7 +77,7 @@ static int set_sock_timeout(int sock, int sec)
     tv.tv_usec = 0;
 
     if (setsockopt(sock, SOL_SOCKET, SO_RCVTIMEO, (char*) &tv, sizeof(tv))) {
-        MOTION_LOG(ERR, TYPE_STREAM, SHOW_ERRNO, "%s: set socket timeout failed");
+        MOTION_LOG(ERR, TYPE_STREAM, SHOW_ERRNO, "set socket timeout failed");
         return 1;
     }
     return 0;
@@ -103,7 +126,7 @@ static int read_http_request(int sock, char* buffer, int buflen, char* uri, int
         nread += readb;
 
         if (nread > buflen) {
-            MOTION_LOG(ERR, TYPE_STREAM, SHOW_ERRNO, "%s: motion-stream End buffer reached"
+            MOTION_LOG(ERR, TYPE_STREAM, SHOW_ERRNO, "motion-stream End buffer reached"
                       " waiting for buffer ending");
             break;
         }
@@ -121,7 +144,7 @@ static int read_http_request(int sock, char* buffer, int buflen, char* uri, int
             return 0;
         }
 
-        MOTION_LOG(ERR, TYPE_STREAM, SHOW_ERRNO, "%s: motion-stream READ give up!");
+        MOTION_LOG(ERR, TYPE_STREAM, SHOW_ERRNO, "motion-stream READ give up!");
         return 0;
     }
 
@@ -179,9 +202,7 @@ static void* handle_basic_auth(void* param)
         "Pragma: no-cache\r\n"
         "WWW-Authenticate: Basic realm=\""STREAM_REALM"\"\r\n\r\n";
 
-#ifdef HAVE_PTHREAD_SETNAME_NP
-    pthread_setname_np(pthread_self(), "handle_basic_auth");
-#endif
+    MOTION_PTHREAD_SETNAME("handle_basic_auth");
 
     pthread_mutex_lock(&stream_auth_mutex);
     p->thread_count++;
@@ -211,14 +232,17 @@ static void* handle_basic_auth(void* param)
 
         authentication = mymalloc(BASE64_LENGTH(auth_size) + 1);
         userpass = mymalloc(auth_size + 4);
-        /* base64_encode can read 3 bytes after the end of the string, initialize it. */
+        /* motion_base64_encode can read 3 bytes after the end of the string, initialize it. */
         memset(userpass, 0, auth_size + 4);
         strcpy(userpass, p->conf->stream_authentication);
-        base64_encode(userpass, authentication, auth_size);
+        motion_base64_encode(userpass, authentication, auth_size);
         free(userpass);
 
         if (strcmp(auth, authentication)) {
             free(authentication);
+            char host[NI_MAXHOST] = "unknown";
+            get_host(host, p->sock);
+            MOTION_LOG(ALR, TYPE_STREAM, NO_ERRNO, "motion-stream - failed auth attempt from %s", host);
             goto Error;
         }
         free(authentication);
@@ -228,15 +252,15 @@ static void* handle_basic_auth(void* param)
 
     /* Set socket to non blocking */
     if (fcntl(p->sock, F_SETFL, p->sock_flags) < 0) {
-        MOTION_LOG(ERR, TYPE_STREAM, SHOW_ERRNO, "%s: fcntl");
+        MOTION_LOG(ERR, TYPE_STREAM, SHOW_ERRNO, "fcntl");
         goto Error;
     }
 
     /* Lock the mutex */
     pthread_mutex_lock(&stream_auth_mutex);
 
-    stream_add_client(&p->cnt->stream, p->sock);
-    p->cnt->stream_count++;
+    stream_add_client(p->stm, p->sock);
+    (*p->stream_count)++;
     p->thread_count--;
 
     /* Unlock the mutex */
@@ -247,7 +271,7 @@ static void* handle_basic_auth(void* param)
 
 Error:
     if (write(p->sock, request_auth_response_template, strlen (request_auth_response_template)) < 0)
-        MOTION_LOG(DBG, TYPE_STREAM, SHOW_ERRNO, "%s: write failure 1:handle_basic_auth");
+        MOTION_LOG(DBG, TYPE_STREAM, SHOW_ERRNO, "write failure 1:handle_basic_auth");
 
 Invalid_Request:
     close(p->sock);
@@ -436,9 +460,7 @@ static void* handle_md5_digest(void* param)
         "</body>\n"
         "</html>\n";
 
-#ifdef HAVE_PTHREAD_SETNAME_NP
-    pthread_setname_np(pthread_self(), "handle_md5_digest");
-#endif
+    MOTION_PTHREAD_SETNAME("handle_md5_digest");
 
     pthread_mutex_lock(&stream_auth_mutex);
     p->thread_count++;
@@ -451,13 +473,13 @@ static void* handle_md5_digest(void* param)
     snprintf(server_nonce, SERVER_NONCE_LEN, "%08x%08x", rand1, rand2);
 
     if (!p->conf->stream_authentication) {
-        MOTION_LOG(ERR, TYPE_STREAM, SHOW_ERRNO, "%s: Error no authentication data");
+        MOTION_LOG(ERR, TYPE_STREAM, SHOW_ERRNO, "Error no authentication data");
         goto InternalError;
     }
     h = strstr(p->conf->stream_authentication, ":");
 
     if (!h) {
-        MOTION_LOG(ERR, TYPE_STREAM, SHOW_ERRNO, "%s: Error no authentication data (no ':' found)");
+        MOTION_LOG(ERR, TYPE_STREAM, SHOW_ERRNO, "Error no authentication data (no ':' found)");
         goto InternalError;
     }
 
@@ -465,7 +487,7 @@ static void* handle_md5_digest(void* param)
     server_pass = (char*)malloc(strlen(h) + 1);
 
     if (!server_user || !server_pass) {
-        MOTION_LOG(ERR, TYPE_STREAM, SHOW_ERRNO, "%s: Error malloc failed");
+        MOTION_LOG(ERR, TYPE_STREAM, SHOW_ERRNO, "Error malloc failed");
         goto InternalError;
     }
 
@@ -567,8 +589,14 @@ static void* handle_md5_digest(void* param)
         DigestCalcHA1((char*)"md5", server_user, (char*)STREAM_REALM, server_pass, (char*)server_nonce, (char*)NULL, HA1);
         DigestCalcResponse(HA1, server_nonce, NULL, NULL, (char*)"", (char*)"GET", server_uri, HA2, server_response);
 
-        if (strcmp(server_response, response) == 0)
+        if (strcmp(server_response, response) == 0){
             break;
+        } else {
+            char host[NI_MAXHOST] = "unknown";
+            get_host(host, p->sock);
+            MOTION_LOG(ALR, TYPE_STREAM, NO_ERRNO, "motion-stream - failed auth attempt from %s", host);
+        }
+
 Error:
         rand1 = (unsigned int)(42000000.0 * rand() / (RAND_MAX + 1.0));
         rand2 = (unsigned int)(42000000.0 * rand() / (RAND_MAX + 1.0));
@@ -577,20 +605,20 @@ Error:
                 "Content-Type: text/html\r\n"
                 "Keep-Alive: timeout=%i\r\n"
                 "Connection: keep-alive\r\n"
-                "Content-Length: %Zu\r\n\r\n",
+                "Content-Length: %zu\r\n\r\n",
                 request_auth_response_template, server_nonce,
                 KEEP_ALIVE_TIMEOUT, strlen(auth_failed_html_template));
         if (write(p->sock, buffer, strlen(buffer)) < 0)
-            MOTION_LOG(DBG, TYPE_STREAM, SHOW_ERRNO, "%s: write failure 1:handle_md5_digest");
+            MOTION_LOG(DBG, TYPE_STREAM, SHOW_ERRNO, "write failure 1:handle_md5_digest");
         if (write(p->sock, auth_failed_html_template, strlen(auth_failed_html_template)) < 0)
-            MOTION_LOG(DBG, TYPE_STREAM, SHOW_ERRNO, "%s: write failure 2:handle_md5_digest");
+            MOTION_LOG(DBG, TYPE_STREAM, SHOW_ERRNO, "write failure 2:handle_md5_digest");
     }
 
     // OK - Access
 
     /* Set socket to non blocking */
     if (fcntl(p->sock, F_SETFL, p->sock_flags) < 0) {
-        MOTION_LOG(ERR, TYPE_STREAM, SHOW_ERRNO, "%s: fcntl");
+        MOTION_LOG(ERR, TYPE_STREAM, SHOW_ERRNO, "fcntl");
         goto Error;
     }
 
@@ -600,8 +628,8 @@ Error:
     /* Lock the mutex */
     pthread_mutex_lock(&stream_auth_mutex);
 
-    stream_add_client(&p->cnt->stream, p->sock);
-    p->cnt->stream_count++;
+    stream_add_client(p->stm, p->sock);
+    (*p->stream_count)++;
 
     p->thread_count--;
     /* Unlock the mutex */
@@ -615,7 +643,7 @@ InternalError:
     free(server_pass);
 
     if (write(p->sock, internal_error_template, strlen(internal_error_template)) < 0)
-      MOTION_LOG(DBG, TYPE_STREAM, SHOW_ERRNO, "%s: write failure 3:handle_md5_digest");
+      MOTION_LOG(DBG, TYPE_STREAM, SHOW_ERRNO, "write failure 3:handle_md5_digest");
 
 Invalid_Request:
     close(p->sock);
@@ -633,7 +661,7 @@ Invalid_Request:
  *
  *
  */
-static void do_client_auth(struct context *cnt, int sc)
+static void do_client_auth(struct context *cnt, struct stream *stm, int *stream_count, int sc)
 {
     pthread_t thread_id;
     pthread_attr_t attr;
@@ -658,26 +686,28 @@ static void do_client_auth(struct context *cnt, int sc)
       handle_func = handle_md5_digest;
       break;
     default:
-      MOTION_LOG(ERR, TYPE_STREAM, SHOW_ERRNO, "%s: Error unknown stream authentication method");
+      MOTION_LOG(ERR, TYPE_STREAM, SHOW_ERRNO, "Error unknown stream authentication method");
       goto Error;
       break;
     }
 
     handle_param = mymalloc(sizeof(struct auth_param));
     handle_param->cnt = cnt;
+    handle_param->stm = stm;
+    handle_param->stream_count = stream_count;
     handle_param->sock = sc;
     handle_param->conf = &cnt->conf;
     handle_param->thread_count = &thread_count;
 
     /* Set socket to blocking */
     if ((flags = fcntl(sc, F_GETFL, 0)) < 0) {
-        MOTION_LOG(ERR, TYPE_STREAM, SHOW_ERRNO, "%s: fcntl");
+        MOTION_LOG(ERR, TYPE_STREAM, SHOW_ERRNO, "fcntl");
         goto Error;
     }
     handle_param->sock_flags = flags;
 
     if (fcntl(sc, F_SETFL, flags & (~O_NONBLOCK)) < 0) {
-        MOTION_LOG(ERR, TYPE_STREAM, SHOW_ERRNO, "%s: fcntl");
+        MOTION_LOG(ERR, TYPE_STREAM, SHOW_ERRNO, "fcntl");
         goto Error;
     }
 
@@ -685,18 +715,18 @@ static void do_client_auth(struct context *cnt, int sc)
         goto Error;
 
     if (pthread_attr_init(&attr)) {
-        MOTION_LOG(ERR, TYPE_STREAM, SHOW_ERRNO, "%s: Error pthread_attr_init");
+        MOTION_LOG(ERR, TYPE_STREAM, SHOW_ERRNO, "Error pthread_attr_init");
         goto Error;
     }
 
     if (pthread_create(&thread_id, &attr, handle_func, handle_param)) {
-        MOTION_LOG(ERR, TYPE_STREAM, SHOW_ERRNO, "%s: Error pthread_create");
+        MOTION_LOG(ERR, TYPE_STREAM, SHOW_ERRNO, "Error pthread_create");
         goto Error;
     }
     pthread_detach(thread_id);
 
     if (pthread_attr_destroy(&attr))
-        MOTION_LOG(ERR, TYPE_STREAM, SHOW_ERRNO, "%s: Error pthread_attr_destroy");
+        MOTION_LOG(ERR, TYPE_STREAM, SHOW_ERRNO, "Error pthread_attr_destroy");
 
     return;
 
@@ -717,15 +747,33 @@ Error:
  */
 int http_bindsock(int port, int local, int ipv6_enabled)
 {
-    int sd = socket(ipv6_enabled?AF_INET6:AF_INET, SOCK_STREAM, 0);
+    int sd = socket(ipv6_enabled?AF_INET6:AF_INET, SOCK_STREAM, IPPROTO_TCP);
+
+    if (sd == -1)
+    {
+        MOTION_LOG(CRT, TYPE_STREAM, SHOW_ERRNO, "error creating socket");
+        return -1;
+    }
 
     int yes = 1, no = 0;
-    setsockopt(sd, SOL_SOCKET, SO_REUSEADDR, &yes, sizeof(yes));
+    if (setsockopt(sd, SOL_SOCKET, SO_REUSEADDR, &yes, sizeof(yes)) != 0)
+    {
+        MOTION_LOG(CRT, TYPE_STREAM, SHOW_ERRNO, "setting SO_REUSEADDR to yes failed");
+        /* we can carry on even if this failed */
+    }
+
     if (ipv6_enabled)
-        setsockopt(sd, IPPROTO_IPV6, IPV6_V6ONLY, &no, sizeof(no));
+    {
+        if (setsockopt(sd, IPPROTO_IPV6, IPV6_V6ONLY, &no, sizeof(no)) != 0)
+        {
+            MOTION_LOG(CRT, TYPE_STREAM, SHOW_ERRNO, "setting IPV6_V6ONLY to no failed");
+            /* we can carry on even if this failed */
+        }
+    }
 
     const char *addr_str;
     struct sockaddr_storage sin;
+    socklen_t sinsize;
     bzero(&sin, sizeof(struct sockaddr_storage));
     sin.ss_family = ipv6_enabled?AF_INET6:AF_INET;
     if (ipv6_enabled) {
@@ -739,6 +787,7 @@ int http_bindsock(int port, int local, int ipv6_enabled)
             addr_str = "any IPv4/IPv6 address";
             sin6->sin6_addr = in6addr_any;
         }
+        sinsize = sizeof(*sin6);
     } else {
         struct sockaddr_in *sin4 = (struct sockaddr_in*)&sin;
         sin4->sin_family = AF_INET;
@@ -750,21 +799,22 @@ int http_bindsock(int port, int local, int ipv6_enabled)
             addr_str = "any IPv4 address";
             sin4->sin_addr.s_addr = htonl(INADDR_ANY);
         }
+        sinsize = sizeof(*sin4);
     }
 
-    if (bind(sd, (struct sockaddr*)&sin, sizeof(sin)) != 0) {
-        MOTION_LOG(NTC, TYPE_STREAM, NO_ERRNO, "%s: error binding on %s port %d", addr_str, port);
+    if (bind(sd, (struct sockaddr*)&sin, sinsize) != 0) {
+        MOTION_LOG(CRT, TYPE_STREAM, SHOW_ERRNO, "error binding on %s port %d", addr_str, port);
         close(sd);
         return -1;
     }
 
     if (listen(sd, DEF_MAXWEBQUEUE) != 0) {
-        MOTION_LOG(CRT, TYPE_STREAM, SHOW_ERRNO, "%s: error listening");
+        MOTION_LOG(CRT, TYPE_STREAM, SHOW_ERRNO, "error listening");
         close(sd);
         return -1;
     }
 
-    MOTION_LOG(NTC, TYPE_STREAM, NO_ERRNO, "%s: listening on %s port %d", addr_str, port);
+    MOTION_LOG(NTC, TYPE_STREAM, NO_ERRNO, "listening on %s port %d", addr_str, port);
 
     return sd;
 }
@@ -783,7 +833,7 @@ static int http_acceptsock(int sl)
     sc = accept(sl, (struct sockaddr*)&addr, &addr_len);
 
     if (sc < 0) {
-        MOTION_LOG(CRT, TYPE_STREAM, SHOW_ERRNO, "%s: motion-stream accept()");
+        MOTION_LOG(CRT, TYPE_STREAM, SHOW_ERRNO, "motion-stream accept()");
         return -1;
     }
 
@@ -941,7 +991,7 @@ static void stream_add_client(struct stream *list, int sc)
     new->socket = sc;
 
     if ((new->tmpbuffer = stream_tmpbuffer(sizeof(header))) == NULL) {
-        MOTION_LOG(ERR, TYPE_STREAM, SHOW_ERRNO, "%s: Error creating tmpbuffer in stream_add_client");
+        MOTION_LOG(ERR, TYPE_STREAM, SHOW_ERRNO, "Error creating tmpbuffer in stream_add_client");
     } else {
         memcpy(new->tmpbuffer->ptr, header, sizeof(header)-1);
         new->tmpbuffer->size = sizeof(header)-1;
@@ -1016,13 +1066,13 @@ static int stream_check_write(struct stream *list)
  *
  * Returns: stream socket descriptor.
  */
-int stream_init(struct context *cnt)
+int stream_init(struct stream *stm, int stream_port, int stream_localhost, int ipv6_enabled)
 {
-    cnt->stream.socket = http_bindsock(cnt->conf.stream_port, cnt->conf.stream_localhost,
-                                       cnt->conf.ipv6_enabled);
-    cnt->stream.next = NULL;
-    cnt->stream.prev = NULL;
-    return cnt->stream.socket;
+    stm->socket = http_bindsock(stream_port, stream_localhost, ipv6_enabled);
+    stm->next = NULL;
+    stm->prev = NULL;
+
+    return stm->socket;
 }
 
 /**
@@ -1030,16 +1080,17 @@ int stream_init(struct context *cnt)
  *      This function is called from the motion_loop when it ends
  *      and motion is terminated or restarted.
  */
-void stream_stop(struct context *cnt)
+void stream_stop(struct stream *stm)
 {
     struct stream *list;
-    struct stream *next = cnt->stream.next;
+    struct stream *next = stm->next;
 
-    MOTION_LOG(NTC, TYPE_STREAM, NO_ERRNO, "%s: Closing motion-stream listen socket"
+    /* TODO friendly info which socket is closing */
+    MOTION_LOG(NTC, TYPE_STREAM, NO_ERRNO, "Closing motion-stream listen socket"
                " & active motion-stream sockets");
 
-    close(cnt->stream.socket);
-    cnt->stream.socket = -1;
+    close(stm->socket);
+    stm->socket = -1;
 
     while (next) {
         list = next;
@@ -1054,7 +1105,7 @@ void stream_stop(struct context *cnt)
         free(list);
     }
 
-    MOTION_LOG(NTC, TYPE_STREAM, NO_ERRNO, "%s: Closed motion-stream listen socket"
+    MOTION_LOG(NTC, TYPE_STREAM, NO_ERRNO, "Closed motion-stream listen socket"
                " & active motion-stream sockets");
 }
 
@@ -1074,12 +1125,13 @@ void stream_stop(struct context *cnt)
  *      Note: Clients that have disconnected are handled in the stream_flush()
  *          function.
  */
-void stream_put(struct context *cnt, unsigned char *image)
+void stream_put(struct context *cnt, struct stream *stm, int *stream_count, unsigned char *image,
+            int do_scale_down)
 {
     struct timeval timeout;
     struct stream_buffer *tmpbuffer;
     fd_set fdread;
-    int sl = cnt->stream.socket;
+    int sl = stm->socket;
     int sc;
     /* Tthe following string has an extra 16 chars at end for length. */
     const char jpeghead[] = "--BoundaryString\r\n"
@@ -1088,6 +1140,11 @@ void stream_put(struct context *cnt, unsigned char *image)
     int headlength = sizeof(jpeghead) - 1;    /* Don't include terminator. */
     char len[20];    /* Will be used for sprintf, must be >= 16 */
 
+    /* will point either to the original image or a scaled down */
+    unsigned char *img = image;
+    int image_width = cnt->imgs.width;
+    int image_height = cnt->imgs.height;
+    int image_size = cnt->imgs.size_norm;
     /*
      * Timeout struct used to timeout the time we wait for a client
      * and we do not wait at all.
@@ -1095,7 +1152,7 @@ void stream_put(struct context *cnt, unsigned char *image)
     timeout.tv_sec = 0;
     timeout.tv_usec = 0;
     FD_ZERO(&fdread);
-    FD_SET(cnt->stream.socket, &fdread);
+    FD_SET(stm->socket, &fdread);
 
     /*
      * If we have not reached the max number of allowed clients per
@@ -1104,34 +1161,54 @@ void stream_put(struct context *cnt, unsigned char *image)
      * add this to the end of the chain of stream structs that are linked
      * to each other.
      */
-    if ((cnt->stream_count < DEF_MAXSTREAMS) &&
+    if ((*stream_count < DEF_MAXSTREAMS) &&
         (select(sl + 1, &fdread, NULL, NULL, &timeout) > 0)) {
         sc = http_acceptsock(sl);
         if (cnt->conf.stream_auth_method == 0) {
-            stream_add_client(&cnt->stream, sc);
-            cnt->stream_count++;
+            stream_add_client(stm, sc);
+            (*stream_count)++;
         } else  {
-            do_client_auth(cnt, sc);
+            do_client_auth(cnt, stm, stream_count, sc);
         }
     }
 
+    /* if there is no connected clients - nothing to do, return */
+    if (*stream_count <= 0)
+        return;
+
+    /* substream put - scale image down and update pointer to the scaled buffer */
+    if (do_scale_down) {
+        /* TODO for now just scale 50%, better resize image to a config predefined size */
+
+        int origwidth = cnt->imgs.width, origheight = cnt->imgs.height;
+        int subwidth = origwidth/2, subheight = origheight/2;
+        int subsize = subwidth * subheight * 3 / 2;
+
+        /* allocate new buffer and scale image */
+        img = scale_half_yuv420p (origwidth, origheight, img);
+
+        image_width = subwidth;
+        image_height = subheight;
+        image_size = subsize;
+    }
+
     /* Lock the mutex */
     if (cnt->conf.stream_auth_method != 0)
         pthread_mutex_lock(&stream_auth_mutex);
 
 
     /* Call flush to send any previous partial-sends which are waiting. */
-    stream_flush(&cnt->stream, &cnt->stream_count, cnt->conf.stream_limit);
+    stream_flush(stm, stream_count, cnt->conf.stream_limit);
 
     /* Check if any clients have available buffers. */
-    if (stream_check_write(&cnt->stream)) {
+    if (stream_check_write(stm)) {
         /*
          * Yes - create a new tmpbuffer for current image.
          * Note that this should create a buffer which is *much* larger
          * than necessary, but it is difficult to estimate the
          * minimum size actually required.
          */
-        tmpbuffer = stream_tmpbuffer(cnt->imgs.size);
+        tmpbuffer = stream_tmpbuffer(cnt->imgs.size_norm);
 
         /* Check if allocation was ok. */
         if (tmpbuffer) {
@@ -1156,8 +1233,8 @@ void stream_put(struct context *cnt, unsigned char *image)
             wptr += headlength;
 
             /* Create a jpeg image and place into tmpbuffer. */
-            tmpbuffer->size = put_picture_memory(cnt, wptr, cnt->imgs.size, image,
-                                                 cnt->conf.stream_quality);
+            tmpbuffer->size = put_picture_memory(cnt, wptr, image_size, img,
+                                       cnt->conf.stream_quality, image_width, image_height);
 
             /* Fill in the image length into the header. */
             imgsize = sprintf(len, "%9ld\r\n\r\n", tmpbuffer->size);
@@ -1177,9 +1254,9 @@ void stream_put(struct context *cnt, unsigned char *image)
              * And finally put this buffer to all clients with
              * no outstanding data from previous frames.
              */
-            stream_add_write(&cnt->stream, tmpbuffer, cnt->conf.stream_maxrate);
+            stream_add_write(stm, tmpbuffer, cnt->conf.stream_maxrate);
         } else {
-            MOTION_LOG(ERR, TYPE_STREAM, SHOW_ERRNO, "%s: Error creating tmpbuffer");
+            MOTION_LOG(ERR, TYPE_STREAM, SHOW_ERRNO, "Error creating tmpbuffer");
         }
     }
 
@@ -1187,11 +1264,17 @@ void stream_put(struct context *cnt, unsigned char *image)
      * Now we call flush again.  This time (assuming some clients were
      * ready for the new frame) the new data will be written out.
      */
-    stream_flush(&cnt->stream, &cnt->stream_count, cnt->conf.stream_limit);
+    stream_flush(stm, stream_count, cnt->conf.stream_limit);
 
     /* Unlock the mutex */
     if (cnt->conf.stream_auth_method != 0)
         pthread_mutex_unlock(&stream_auth_mutex);
 
+    /* free resized image buffer */
+    if (do_scale_down)
+    {
+        free (img);
+    }
+
     return;
 }
